(function () {
  window.__frontend = {
    version: '5.1.0-alpha.14',
    deps: {
      '@labelu/audio-annotator-react': '1.5.0-alpha.5',
      '@labelu/components-react': '1.4.2-alpha.2',
      '@labelu/formatter': '1.0.2',
      '@labelu/image': '1.1.0',
      '@labelu/image-annotator-react': '2.1.0-alpha.4',
      '@labelu/interface': '1.3.1',
      '@labelu/video-annotator-react': '1.3.4-alpha.5',
      '@labelu/video-react': '1.3.2-alpha.2',
    },
  };
})();
